package net.knarcraft.dynmapcitizens.handler.trait.quests;

import me.blackvein.quests.quests.IQuest;
import me.blackvein.quests.quests.Planner;
import net.knarcraft.dynmapcitizens.util.TimeFormatter;

/**
 * A class to generate a string containing all information about a quest's planner info
 */
public class QuestPlannerInfoGenerator {

    private final IQuest quest;

    /**
     * Instantiates a new quest planner info generator
     *
     * @param quest <p>The quest to generate information about</p>
     */
    public QuestPlannerInfoGenerator(IQuest quest) {
        this.quest = quest;
    }

    /**
     * Gets information about the time-availability of a quest
     *
     * @return <p>Information about when the quest is available</p>
     */
    public String getQuestPlannerInfo() {
        Planner planner = quest.getPlanner();
        StringBuilder plannerInfo = new StringBuilder();
        plannerInfo.append("<b>Planner:</b><ul>");

        //Quest can be repeated after a cool-down
        if (planner.hasCooldown()) {
            plannerInfo.append("<li>Quest repeatable after: ");
            plannerInfo.append(TimeFormatter.getDurationString(planner.getCooldown() / 1000));
            plannerInfo.append("</li>");
        } else {
            plannerInfo.append("<li>Quest cannot be repeated!</li>");
        }

        //Quest only becomes available after the start date
        if (planner.hasStart()) {
            plannerInfo.append("<li>Quest available from ");
            plannerInfo.append(TimeFormatter.formatTimestamp(planner.getStartInMillis())).append("</li>");
        }

        //Quest is only available until the end date
        if (planner.hasEnd()) {
            plannerInfo.append("<li>Quest available until ");
            plannerInfo.append(TimeFormatter.formatTimestamp(planner.getEndInMillis())).append("</li>");
        }

        //Quest availability repeats
        if (planner.hasRepeat()) {
            plannerInfo.append("<li>Quest will become available again after ");
            plannerInfo.append(TimeFormatter.getDurationString(planner.getRepeat() / 1000)).append("</li>");
        }

        plannerInfo.append("</ul>");
        return plannerInfo.toString();
    }

}
